@@ -78,6 +78,9 @@ public:
    int discretizationSample ;
    int bayesDiscretization ;
    int bayesEqFreqIntervals ;
+   int maxValues4Exhaustive ; // maximal values of discrete attribute values to try exhaustive binarization
+   int maxValues4Greedy ; // maximal values of discrete attribute values to try greedy binarization;
+                          // if an attribute has more vaues than this random binarization is tried
 
    // pruning
    int selectedPruner ;
