@@ -13,7 +13,7 @@
 #include "ftree.h"
 #include "options.h"
 
-extern Options *opt ;
+// extern Options *opt ;
 
 //************************************************************
 //
